#include "modulesmanager.h"

#include <QInputDialog>
#include <QMessageBox>
#include <QDir>

#include "Core/module.h"
#include "Core/xmlobject.h"
#include "Modules/Utility/modulespicker.h"
#include "Modules/Utility/saveconfigurationdialog.h"
#include "Modules/skywardhubstrings.h"

ModulesList ModulesManager::modulesListHandler;

ModulesManager::ModulesManager()
{

}

ModulesManager::~ModulesManager()
{
    clearPages();
    for(int i = 0; i < menuActions.count(); i++){
        delete menuActions[i];
    }
    menuActions.clear();
}

QString ModulesManager::getModuleName(ModuleId id)
{
    if(ModulesManager::modulesListHandler.containsId(id))
        return ModulesManager::modulesListHandler.getModuleName(id);

    return "";
}

Module *ModulesManager::instantiateModuleById(ModuleId id)
{
    if(ModulesManager::modulesListHandler.containsId(id))
        return ModulesManager::modulesListHandler.getFactory(id)();

    return nullptr;
}

Module *ModulesManager::instantiateModuleById(ModuleId id, const XmlObject &params)
{
    Module *m = instantiateModuleById(id);

    if(m != nullptr)
        m->initialize(params);
    return m;
}

Module *ModulesManager::instantiateModuleByName(const QString &moduleName)
{
    return ModulesManager::modulesListHandler.findFactoryByModuleName(moduleName)();
}

Module *ModulesManager::instantiateModuleByName(const QString &moduleName, const XmlObject &params)
{
    Module *m = instantiateModuleByName(moduleName);

    if(m != nullptr)
        m->initialize(params);
    return m;
}

QList<Module *>ModulesManager::loadModuleFromXml(XmlObject &xml)
{
    QList<Module*> windows;

    for(int i=0; i < xml.childCount(); i++){
        XmlObject *child = xml.getChild(i);
        QString moduleName = child->getObjectName();
        Module *module = ModulesManager::instantiateModuleByName(moduleName);
        if(module != nullptr){
            module->fromXmlObject(*child);
            windows.append(module);
        }
    }

    return windows;
}

void ModulesManager::setModules(QList<Module *> modules)
{
    int i;
    for(i=0 ; i < pages.count() && i < modules.count(); i++ ){
        Module *old = pages[i];
        emit pages[i]->getModuleEventsHandler()->replaceMeWith(old, modules[i]);
        delete old;
    }
    if(i < pages.count()){
        for(int c = pages.count() - 1; c > i; c-- ){
            delete pages[c];
        }
    }
    for(; i < modules.count(); i++ ){
        addPage(modules[i]);
    }
}

void ModulesManager::addModules(QList<Module *> modules)
{
    for(int i=0; i < modules.count(); i++){
        addPage(modules[i]);
    }
}

void ModulesManager::addPage(Module *module)
{
    pages.append(module);
    connectModule(module);
    emit pageAdded(pages.indexOf(module));
}

void ModulesManager::removePage(Module *module)
{
    int index = pages.indexOf(module);
    if(index >= 0){
        pages.removeAt(index);
        disconnectModule(module);
    }
}

Module *ModulesManager::instantiateDefaultModule()
{        
    return ModulesManager::instantiateModuleById(ModuleId::EMPTY);
}

Module *ModulesManager::invokeModulesPickerService()
{
#ifdef MODULESPICKER_H

    ModulesPicker *p = new ModulesPicker();
    return p->start();

#endif
    return  nullptr;
}

QList<QString> ModulesManager::getModulesNamesList()
{
    return ModulesManager::modulesListHandler.getModulesNamesList();
}


void ModulesManager::connectModule(Module *module)
{
    if(module){
        connect(module->getModuleEventsHandler(),&ModuleEventsHandler::contextMenuRequest,this,&ModulesManager::onContextMenuRequest);
        connect(module->getModuleEventsHandler(),&ModuleEventsHandler::beforeDelete,this,&ModulesManager::onModuleDeleted);
        connect(module->getModuleEventsHandler(),&ModuleEventsHandler::replaceMeWith,this,&ModulesManager::onReplaceMeWith);
    }

    //connect(module->getModuleEventsHandler(),&ModuleEventsHandler::replaceMeWithModules,this,&ModulesManager::openWindows);
}

void ModulesManager::disconnectModule(Module *module)
{
    if(module){
        disconnect(module->getModuleEventsHandler(),&ModuleEventsHandler::contextMenuRequest,this,&ModulesManager::onContextMenuRequest);
        disconnect(module->getModuleEventsHandler(),&ModuleEventsHandler::beforeDelete,this,&ModulesManager::onModuleDeleted);
        disconnect(module->getModuleEventsHandler(),&ModuleEventsHandler::replaceMeWith,this,&ModulesManager::onReplaceMeWith);
    }
}

void ModulesManager::onModuleDeleted(Module *deletedModule)
{
    int index = pages.indexOf(deletedModule);

    if(index == 0){
        onFirstPageDeleted();
    }else{
        removePage(deletedModule);
    }
}

void ModulesManager::openNewEmptyWindow()
{
    addPage(ModulesManager::instantiateDefaultModule());
}

Module *ModulesManager::getModuleAt(int index)
{
    if(index >= 0 && index < pages.count()){
        return pages.at(index);
    }
    return nullptr;
}

int ModulesManager::getModuleCount()
{
    return pages.count();
}

void ModulesManager::clearPages()
{
    for(int i=pages.count() - 1; i >= 0; i--){
        delete pages[i];
    }
    pages.clear();
}

void ModulesManager::onReplaceMeWith(Module *sender, Module *newModule)
{
    if(sender != nullptr && newModule != nullptr){
        int index = pages.indexOf(sender);
        if(index >= 0){
            pages[index] = newModule;
            connectModule(newModule);
            disconnectModule(sender);
        }
    }
}

bool ModulesManager::getRebuild() const
{
    return rebuild;
}

void ModulesManager::setRebuild(bool value)
{
    rebuild = value;
}

void ModulesManager::onContextMenuRequest(QMenu &menu, const QPoint &p)
{
    QMenu hubMenu("Hub Menu");
    hubMenu.addActions(getHubMenuActions());
    menu.addMenu(&hubMenu);
    emit contextMenuRequest(menu, p);
}

QList<QAction*> ModulesManager::getHubMenuActions()
{
    if(menuActions.count() == 0){
        QAction* save = new QAction("Save");
        menuActions.append(save);
        connect(save,&QAction::triggered,this,&ModulesManager::saveConfigurationService);

        QAction* openWindows = new QAction("New Windows");
        menuActions.append(openWindows);
        connect(openWindows,&QAction::triggered,this,&ModulesManager::openNewEmptyWindow);
    }

    return menuActions;
}

QList<ModuleInfo> ModulesManager::getModulesInfo()
{
    return modulesListHandler.getModulesInfo();
}

void ModulesManager::saveConfigurationService()
{
    if(pages.isEmpty()){
        QMessageBox errorMsg;
        errorMsg.setText("No module to be saved found");
        errorMsg.exec();
        return;
    }

    bool ok = false;

#ifdef SAVECONFIGURATIONDIALOG_H
    // Ask configuration data to the user
    SaveConfigurationDialog* saveDialog = new SaveConfigurationDialog();
    saveDialog->setConfigName(SkywardHubStrings::defaultConfigurationFileName);
    saveDialog->setConfigIconPath(SkywardHubStrings::defaultConfigurationIconPath);
    QString configName;
    QString configIconPath;
    QString description;
    saveDialog->pickConfigData(&configName, &configIconPath, &description);
    ok = !configName.isEmpty();
#endif

    XmlObject xml(SkywardHubStrings::configurationNameValue);
    xml.addAttribute(SkywardHubStrings::configurationIconPathAttribute, configIconPath);
    xml.addAttribute(SkywardHubStrings::configurationDescriptionAttribute, description);

    if(ok && !configName.isEmpty()){
        configName = configName.replace(".xml", "");
        xml.addAttribute(SkywardHubStrings::configurationNameAttribute, configName);
        if(configName.indexOf(".xml") < 0 ){
            configName.append(".xml");
        }

        if(!QDir(SkywardHubStrings::defaultPrefabsFolder).exists()){
            QDir().mkpath(SkywardHubStrings::defaultPrefabsFolder);
        }

        for(Module *module : pages){
            xml.addChild(module->toXmlObject());
        }

        QString resultTxt = "Impossible to save";
        if(xml.writeToFile(SkywardHubStrings::defaultPrefabsFolder + configName)){
            resultTxt = "The configuration has been saved in " + configName;
        }
        QMessageBox resultMsg;
        resultMsg.setText(resultTxt);
        resultMsg.exec();
    }
}

void ModulesManager::onFirstPageDeleted()
{
    if(!rebuild)
        return;
    QMessageBox msgBox;
    msgBox.setWindowTitle("title");
    msgBox.setText("Do you want to close the App?");
    msgBox.setStandardButtons(QMessageBox::Yes);
    msgBox.addButton(QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::No);
    if(msgBox.exec() == QMessageBox::No){
        emit pages[0]->getModuleEventsHandler()->replaceMeWith(pages[0], instantiateDefaultModule());
    }
    else{
        emit appQuitRequested();
    }
}

