#include "filestreammodule.h"
#include "ui_filestreammodule.h"

#include "Core/modulemessagesbroker.h"
#include <QDebug>
#include <QMessageBox>
#include "Modules/skywardhubstrings.h"
#include <QDir>
#include <QDesktopServices>
#include <QUrl>

FileStreamModule::FileStreamModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::FileStreamModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    connectUI();

    ui->filePath_lineEdit->setText(SkywardHubStrings::defaultStreamFile);
}

FileStreamModule::~FileStreamModule()
{
    if(file.isOpen())
        file.close();
    deleteAllTopicViews();
    delete ui;
}

QWidget *FileStreamModule::toWidget()
{
    return this;
}

XmlObject FileStreamModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::FILESTREAM));

    if(getFilePath() != "")
        obj.addAttribute("FilePath", getFilePath());

    for(int i = 0; i < topicViewsList.count();i++){
        XmlObject subscription("Subscription");
        subscription.setTextValue(topicViewsList[i]->text().trimmed());
        obj.addChild(subscription);
    }
    return obj;
}

void FileStreamModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getName(ModuleId::FILESTREAM)){

        if(xmlObject.hasAttribute("FilePath")){
            ui->filePath_lineEdit->setText(xmlObject.getAttribute("FilePath"));
        }

        for(int i = 0; i < xmlObject.childCount(); i++){
            XmlObject child = xmlObject.childAt(i);
            if(child.getObjectName() == "Subscription" && child.getTextValue() != ""){
                addTopic(child.getTextValue());
            }
        }
    }
}

void FileStreamModule::connectUI()
{
    connect(ui->start_button, &QPushButton::clicked, this, &FileStreamModule::onStartClicked);
    connect(ui->stop_button, &QPushButton::clicked, this, &FileStreamModule::onStopClicked);
    connect(ui->addTopic_button, &QPushButton::clicked, this, &FileStreamModule::onAddTopicClicked);
    connect(ui->showFile_button, &QPushButton::clicked, this, &FileStreamModule::onShowFileClick);
}

QString FileStreamModule::getFilePath() const
{
    return ui->filePath_lineEdit->text().trimmed();
}

QString FileStreamModule::getTopicName() const
{
    return ui->topicName_lineEdit->text().trimmed();
}

void FileStreamModule::resetTopicName()
{
    ui->topicName_lineEdit->clear();
}

void FileStreamModule::onMsgReceived(const ModuleMessage &msg)
{
//    if(file.isOpen() ){
//        QTextStream out(&file);
//        out << msg.payload() << Qt::endl;
//    }

    QFile file(getFilePath());
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    QTextStream out(&file);
    out << msg.payload() << Qt::endl;
}

void FileStreamModule::onAddTopicClicked()
{
    QString topicName = getTopicName();
    resetTopicName();
    if(topicName != ""){
        addTopic(topicName);
    }
}

void FileStreamModule::onStartClicked()
{
    file.setFileName(getFilePath());
    if(!file.open(QIODevice::WriteOnly | QIODevice::Text)){ // Append QIODevice::ReadWrite
        QString msg = "FileStream: Error occurred while opening file " + getFilePath();
        QMessageBox msgBox;
        msgBox.setText(msg);
        msgBox.exec();
        return;
    }

    file.resize(0);
    disableOnStartElement();
    for(int i = 0; i < topicViewsList.count() ; i++ ){
        getCore()->getModuleMessagesBroker()->subscribe(topicViewsList[i]->text().trimmed(), this, [this](const ModuleMessage &msg){
            onMsgReceived(msg);
        });
        topicViewsList[i]->setEnabled(false);
    }

}

void FileStreamModule::onStopClicked()
{
    if(file.isOpen())
        file.close();
    enableElementOnStop();

    for(int i = 0; i < topicViewsList.count(); i++ ){
        getCore()->getModuleMessagesBroker()->unsubscribe(topicViewsList[i]->text().trimmed(), this);
        topicViewsList[i]->setEnabled(true);
    }
}

void FileStreamModule::onTopicEdit()
{
    for(int i = topicViewsList.count() - 1 ; i >= 0; i-- ){
        if(topicViewsList[i]->text().trimmed() == ""){
            topicViewsList[i]->deleteLater();
            topicViewsList.removeAt(i);
        }
    }
}

void FileStreamModule::disableOnStartElement()
{
    ui->start_button->setEnabled(false);
    ui->addTopic_button->setEnabled(false);
    ui->filePath_lineEdit->setEnabled(false);
}

void FileStreamModule::enableElementOnStop()
{
    ui->start_button->setEnabled(true);
    ui->addTopic_button->setEnabled(true);
    ui->filePath_lineEdit->setEnabled(true);
    ui->writeOnFile_radioButton->setEnabled(true);
}

void FileStreamModule::onShowFileClick()
{
    QDir dir(SkywardHubStrings::defaultLogsFolder);
    if (dir.exists()){
        QDesktopServices::openUrl( QUrl::fromLocalFile(getFilePath()) );
    }
    else{
        QString msg = "The file does not exist." +  getFilePath();
        QMessageBox msgBox;
        msgBox.setText(msg);
        msgBox.exec();
    }
}

void FileStreamModule::addTopic(const QString &topic)
{
    QLineEdit* topicView = createTopicView(topic);

    topicViewsList.append(topicView);
    ui->topics_layout->addWidget(topicView);
}

QLineEdit *FileStreamModule::createTopicView(const QString &topic) const
{
    QLineEdit* topicView = new QLineEdit();
    topicView->setText(topic);

    connect(topicView, &QLineEdit::textEdited, this, &FileStreamModule::onTopicEdit);
    return topicView;
}

void FileStreamModule::deleteAllTopicViews()
{
    for(int i = 0; i < topicViewsList.count(); i++ ){
        getCore()->getModuleMessagesBroker()->unsubscribe(topicViewsList[i]->text().trimmed(), this);
        ui->topics_layout->removeWidget(topicViewsList[i]);
        topicViewsList[i]->deleteLater();
    }
    topicViewsList.clear();
}

