#include "graphmodule.h"
#include "ui_graphmodule.h"

#include "Modules/Utility/contextmenuseparator.h"
#include "Core/modulemessagesbroker.h"
#include "Modules/Utility/subscriptionspanel.h"
#include <QDebug>
#include <QTimer>

GraphModule::GraphModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::GraphModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    buildCentralGraphView();
    ui->centralLayout->addWidget(&graphCentralView);
    connect(&updaterTimer, &QTimer::timeout, this, &GraphModule::onUpdateTimerTick);
    updaterTimer.setSingleShot(false);
    updaterTimer.start(updatePeriod);
}

GraphModule::~GraphModule()
{
    // QCPGraph are destroyed automatically

    subscriptions.clear();
    delete ui;
}

void GraphModule::buildCentralGraphView()
{
    setTheme();

    graphCentralView.yAxis2->setVisible(true);
    graphCentralView.yAxis2->setTicks(false);
    graphCentralView.yAxis2->setTickLabels(false);

    //graphCentralView.xAxis2->setVisible(true);
    //textTicker = QSharedPointer<QCPAxisTickerText>(new QCPAxisTickerText());
    //cPlot.xAxis2->setTicker(textTicker);
    //graphCentralView.xAxis2->setTickLabels(true);

    QSharedPointer<QCPAxisTickerTime> dateTimeTicker(new QCPAxisTickerTime);
    graphCentralView.xAxis->setTicker(dateTimeTicker);
    //QDateTime rangeLowerBound = QDateTime::currentDateTime().addDays(-2);
//    double now = QDateTime::currentDateTime().toTime_t();
//    QDateTime rangeUpperBound = QDateTime::currentDateTime().addDays(+2);
//    graphCentralView.xAxis->setRange(now, QCPAxisTickerDateTime::(rangeUpperBound));
    dateTimeTicker->setTimeFormat(dateFormat);

    graphCentralView.legend->setVisible(true);

    // make left and bottom axes always transfer their ranges to right and top axes:
    connect(graphCentralView.xAxis, SIGNAL(rangeChanged(QCPRange)), graphCentralView.xAxis2, SLOT(setRange(QCPRange)));
    connect(graphCentralView.yAxis, SIGNAL(rangeChanged(QCPRange)), graphCentralView.yAxis2, SLOT(setRange(QCPRange)));

    // Allow user to drag axis ranges with mouse, zoom with mouse wheel and select graphs by clicking:
    graphCentralView.setInteractions(QCP::iRangeDrag | QCP::iRangeZoom | QCP::iSelectPlottables);

    // Enable context menu
    graphCentralView.setContextMenuPolicy(Qt::CustomContextMenu);
    //connect(&graphCentralView,SIGNAL(customContextMenuRequested(QPoint)),this,SLOT(onCustomContextMenuRequested(QPoint)));
    connect(&graphCentralView, &QCustomPlot::customContextMenuRequested, this, &GraphModule::onCustomContextMenuRequested);
    //connect(customPlot,SIGNAL(selectionChangedByUser()),this,SLOT(selectedGraphChange()));
}

void GraphModule::setTheme()
{
//    QColor accentColor;
//    QColor themeColor;
//    QColor themeBackgroundColor;
//    QColor traceColor;

//    accentColor = QColor(78, 156, 207);
//    traceColor = accentColor;
//    themeColor = QColor(Qt::gray);
//    themeBackgroundColor = QColor(50, 50, 50);

//    foreach(QCPAxisRect* rect, graphCentralView.axisRects()){
//        foreach(QCPAxis* axis, rect->axes()){
//            axis->setLabelColor(themeColor);
//            axis->setBasePen(QPen(themeColor, 1));
//            axis->setTickPen(QPen(themeColor, 1));
//            axis->setSubTickPen(QPen(themeColor, 1));
//            axis->setTickLabelColor(themeColor);
//            axis->grid()->setPen(QPen(themeColor, 0.5, Qt::DotLine));
//            axis->grid()->setSubGridVisible(false);

//            axis->setSelectedTickLabelColor(accentColor);
//            axis->setSelectedLabelColor(accentColor);
//            axis->setSelectedBasePen(QPen(accentColor, 1));
//            axis->setSelectedSubTickPen(QPen(accentColor, 1));
//            axis->setSelectedTickPen(QPen(accentColor, 1));
//        }
//    }
//    graphCentralView.setBackground(QBrush(themeBackgroundColor));
    //    replotAll();
}

void GraphModule::onUpdateTimerTick()
{
    replotAll();
}

QCPGraph *GraphModule::instantiateNewGraph()
{
    QCPGraph *graph = graphCentralView.addGraph();
    int r = rand() % 255;
    int g = rand() % 255;
    int b = rand() % 255;
    graph->setPen(QPen(QColor(r, g, b), 2));
    graph->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssCircle, 3));
    graph->setSelectable(QCP::stSingleData);
    return graph;
}


QWidget *GraphModule::toWidget()
{
    return this;
}

XmlObject GraphModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::GRAPH));
    QList<QString> subscriptionsList = subscriptions.keys();
    for(int i = 0; i < subscriptionsList.count();i++){
        XmlObject subscription("Subscription");
        subscription.addAttribute("Value",subscriptionsList[i]);
        obj.addChild(subscription);
    }
    return obj;
}

void GraphModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getName(ModuleId::GRAPH)){
        for(int i = 0; i < xmlObject.childCount(); i++){
            XmlObject child = xmlObject.childAt(i);
            if(child.getObjectName() == "Subscription"){
                QString subscription = child.getAttribute("Value");
                if(subscription != ""){
                    onSubscriptionAdded(subscription);
                }
            }
        }
    }
}

void GraphModule::onSubscribeClicked()
{
    QList<QString> subs = subscriptions.keys();
    SubscriptionsPanel *sPanel = new SubscriptionsPanel(&subs);
    sPanel->setWindowTitle("Graph Subscriptions");
    connect(sPanel, &SubscriptionsPanel::SubscriptionAdded, this, &GraphModule::onSubscriptionAdded);
    connect(sPanel, &SubscriptionsPanel::SubscriptionRemoved, this, &GraphModule::onSubscriptionRemoved);
    sPanel->show();
}


void GraphModule::onSubscriptionAdded(const QString &subscription)
{
    QCPGraph *graph = instantiateNewGraph();
    graph->setName(subscription);
    subscriptions[subscription] = graph;
    getCore()->getModuleMessagesBroker()->subscribe(subscription,this,[this](const ModuleMessage &msg){
        onMsgReceived(msg);
    });


    //Da rimuovere per performance
    graphCentralView.replot();
}

void GraphModule::onSubscriptionRemoved(const QString &subscription)
{
    QCPGraph *graph = subscriptions[subscription];
    graphCentralView.removeGraph(graph);
    subscriptions.remove(subscription);
    getCore()->getModuleMessagesBroker()->unsubscribe(subscription, this);

    //Da rimuovere per performance
    graphCentralView.replot();
}

void GraphModule::onMsgReceived(const ModuleMessage &msg)
{
    if(!subscriptions.contains(msg.originalTopic())){
        onSubscriptionAdded(msg.originalTopic());
    }

    if(stopPlot)
        return;

    if(subscriptions.contains(msg.originalTopic())){
        QCPGraph *graph = subscriptions[msg.originalTopic()];

        double x = 0, y = 0;
        x = msg.timestamp().msecsSinceStartOfDay();
        bool ok;
        y = msg.payload().toDouble(&ok);
        if(ok && graph != nullptr){
            graph->addData(x,y);
            //Da rimuovere per performance
            //replotAll();
        }
    }
}

void GraphModule::setFollowedGraphIndex(bool checked)
{
    if(checked){
        QCPGraph* selectedGraph = getSelectedGraph();

        if(selectedGraph){

            for(int i = 0; i < graphCentralView.graphCount(); i++){
                if(selectedGraph == graphCentralView.graph(i))
                    followedGraphIndex = i;
            }
        }
        else if(graphCentralView.graphCount() > 0){
            // If no graph is selected, select the first graph
            followedGraphIndex = 0;
        }
    }
    else{
        followedGraphIndex = -1;
    }
}

void GraphModule::onClearClicked()
{
    QMapIterator<QString,QCPGraph*> i(subscriptions);
    while (i.hasNext()) {
        i.next();
        i.value()->data()->clear();
    }

    replotAll();
}

void GraphModule::onStopClicked(bool checked)
{
    stopPlot = checked;

    if(stopPlot)
        updaterTimer.stop();
    else{
        updaterTimer.start(updatePeriod);
    }
}


QCPGraph *GraphModule::getSelectedGraph()
{
    QList<QCPGraph*> selection;
    //if(graph)
    selection = graphCentralView.selectedGraphs();
    if(selection.size()>0){
        return selection.first();
    }
    return nullptr;
}

void GraphModule::centerView(const QCPGraph * graph)
{
    if(!graph->data()->isEmpty()){
        double lastKey = (graph->data()->constEnd()-1)->key;
        double lastValue = (graph->data()->constEnd()-1)->value;
        double size_x = graphCentralView.xAxis->range().size();
        double size_y = graphCentralView.yAxis->range().size();
        graphCentralView.xAxis->setRange(lastKey, size_x, Qt::AlignmentFlag::AlignCenter);
        graphCentralView.yAxis->setRange(lastValue, size_y, Qt::AlignmentFlag::AlignCenter);
    }
}

void GraphModule::replotAll()
{
    if(followedGraphIndex < graphCentralView.graphCount() && followedGraphIndex >= 0){
        centerView(graphCentralView.graph(followedGraphIndex));
    }
    graphCentralView.replot();
}

void GraphModule::addCustomActionsToMenu()
{
    QAction* subscribe = new QAction("Subscribe");
    connect(subscribe, &QAction::triggered,this, &GraphModule::onSubscribeClicked);

    QAction* clear = new QAction("Clear");
    connect(clear, &QAction::triggered,this, &GraphModule::onClearClicked);

    QAction* stop = new QAction("Stop");
    stop->setCheckable(true);
    stop->setChecked(stopPlot);
    connect(stop,&QAction::triggered,this,&GraphModule::onStopClicked);

    QAction* follow = new QAction("Follow");
    follow->setCheckable(true);
    if(followedGraphIndex >= 0){
        follow->setChecked(true);
    }
    connect(follow,&QAction::triggered,this,&GraphModule::setFollowedGraphIndex);

    addActionToMenu(follow);
    addActionToMenu(subscribe);
    addActionToMenu(clear);
    addActionToMenu(stop);
}

