#include "imagevieweritem.h"

#include <QSize>


ImageViewerItem::ImageViewerItem()
{
    initialize();
}

ImageViewerItem::ImageViewerItem(const QPointF &center, const QSize &value) : ImageViewerItem()
{
    setSize(value);
    setCenter(center);
}

ImageViewerItem::ImageViewerItem(const ImageViewerItem &other) : ImageViewerItem()
{
    copy(other);
}

QString ImageViewerItem::getText() const
{
    XmlObject text = xml.searchChild("text");
    if(text.isEmpty())
        return defaultText;
    return text.getTextValue();
}


XmlObject *ImageViewerItem::findOrAddXmlObject(const QString &xmlChildName)
{
    QList<XmlObject*> childs = xml.deepSearchObjects(xmlChildName);
    if(childs.isEmpty()){
        XmlObject c(xmlChildName);
        int childIndex = xml.addChild(c);
        return xml.getChild(childIndex);
    }
    else{
        return childs[0];
    }
}

void ImageViewerItem::setText(const QString &value)
{
    XmlObject *text = findOrAddXmlObject("text");
    text->setTextValue(value);
}

void ImageViewerItem::setCenter(const QPointF &center)
{
    float topX = 0;
    float topY = 0;

    topX = center.x() - (getSize().width() / 2);
    topY = center.y() - (getSize().height() / 2);

    setTopLeftCornerPosition(QPointF(topX, topY));
}

int ImageViewerItem::getId() const
{
    return id;
}

void ImageViewerItem::setId(int value)
{
    id = value;
}

XmlObject ImageViewerItem::toXmlObject() const
{
    XmlObject xmlObj = xml;
    xmlObj.addAttribute("topX", getTopLeftCornerPosition().x());
    xmlObj.addAttribute("topY", getTopLeftCornerPosition().y());
    return xmlObj;
}

void ImageViewerItem::operator =(const ImageViewerItem &other)
{
    copy(other);
}

bool ImageViewerItem::onMouseMoved(const QPointF &point)
{
    bool needUpdate = false;
//    if(isPointInsideMyArea(point)){
//        setClicked(true);
//        needUpdate = true;
//    }
//    else{
//        if(getClicked()){
//            setClicked(false);
//            needUpdate = true;
//        }
//    }

    if(isDragActive){
        QPointF newTopPosition(point.x() - dragOffset.x(), point.y() - dragOffset.y());
        topLeftCornerPosition = newTopPosition;
        needUpdate = true;
    }

    return needUpdate;
}

bool ImageViewerItem::isPointInsideMyArea(const QPointF &point)
{
    float topX = getTopLeftCornerPosition().x();
    float topY = getTopLeftCornerPosition().y();

    float deltaX = point.x() - topX;
    float deltaY = point.y() - topY;

    if(deltaX >= 0 && deltaX <= getSize().width() &&
       deltaY >= 0 && deltaY <= getSize().height()){
        return true;
    }
    return false;
}

void ImageViewerItem::startDrag(const QPointF &initialPosition)
{
    float offsetX = initialPosition.x() - getTopLeftCornerPosition().x();
    float offsetY = initialPosition.y() - getTopLeftCornerPosition().y();
    dragOffset = QPointF(offsetX, offsetY);
    isDragActive = true;
}

void ImageViewerItem::stopDrag()
{
    isDragActive = false;
}

void ImageViewerItem::onModuleMessageReceived(const ModuleMessage &msg)
{
    setText(msg.payload());
}

QString ImageViewerItem::getTopic() const
{
    XmlObject topic = xml.searchChild("topic");
    return topic.getTextValue();
}

void ImageViewerItem::copy(const ImageViewerItem &other)
{
    setId(other.getId());
    setText(other.getText());
    setTopLeftCornerPosition(other.getTopLeftCornerPosition());
    setXml(other.toXmlObject());
}

void ImageViewerItem::setTopLeftCornerPosition(const QPointF &value)
{
    topLeftCornerPosition = value;
}

void ImageViewerItem::setTopic(const QString &topic)
{
    XmlObject *xmlTopic = findOrAddXmlObject("topic");
    xmlTopic->setTextValue(topic);
}

void ImageViewerItem::initialize()
{
    xml.setObjectName("svg");
    QSize s = defaultSize;
    xml.addAttribute("height", QString::number(s.height()));
    xml.addAttribute("width", QString::number(s.width()));

    XmlObject rect("rect");
    rect.addAttribute("height", QString::number(s.height()));
    rect.addAttribute("width", QString::number(s.width()));    
    rect.addAttribute("fill", "none");
    rect.addAttribute("stroke", "#000000");
    rect.addAttribute("stroke-width", "2");

    XmlObject text("text");
    text.addAttribute("x",5);
    text.addAttribute("y",20);
    text.addAttribute("fill", "#000000");
    text.addAttribute("font-size", "20");
    text.setTextValue(defaultText);

    xml.addChild(text);
    xml.addChild(rect);
}

void ImageViewerItem::setXml(const XmlObject &value)
{
    xml = value;

    int height, width;
    if(xml.getIntAttribute("height", height) && xml.getIntAttribute("width", width)){
        setSize(width, height);
    }


    QList<XmlObject*> childs = xml.deepSearchObjects("text");
    if(!childs.isEmpty()){
        XmlObject *text = childs[0];
        setText(text->getTextValue());
    }


    int topx, topy;
    if(value.getIntAttribute("topX", topx) && value.getIntAttribute("topY", topy)){
        setTopLeftCornerPosition(QPointF(topx, topy));
    }
}

//QString ImageViewerItem::getCurrentBorderColor() const
//{
//    if(getClicked())
//        return clickedBorderColor;
//    return defaultBorderColor;
//}

//bool ImageViewerItem::getClicked() const
//{
//    return clicked;
//}

//void ImageViewerItem::setClicked(bool value)
//{
//    clicked = value;
//}

QSize ImageViewerItem::getSize() const
{
    int height, width;
    if(xml.getIntAttribute("height", height) && xml.getIntAttribute("width", width)){
        return QSize(width, height);
    }
    return defaultSize;
}

void ImageViewerItem::setSize(const QSize &value)
{
    xml.addAttribute("height", value.height());
    xml.addAttribute("width", value.width());

    QList<XmlObject*> childs = xml.deepSearchObjects("rect");
    if(!childs.isEmpty()){
        XmlObject *border = childs[0];
        border->addAttribute("height", value.height());
        border->addAttribute("width", value.width());
    }
}

void ImageViewerItem::setSize(int width, int height)
{
    setSize(QSize(width, height));
}

QPointF ImageViewerItem::getTopLeftCornerPosition() const
{
    return topLeftCornerPosition;
}
