#include "imageviewermodule.h"
#include "ui_imageviewermodule.h"

#include <QFile>
#include <QTextStream>
#include <QTextDocument>
#include "Modules/Utility/contextmenuseparator.h"
#include "Core/modulemessagesbroker.h"
#include "Modules/skywardhubstrings.h"
#include <QMouseEvent>
#include <QMessageBox>
#include <QInputDialog>

ImageViewerModule::ImageViewerModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::ImageViewerModule)
{
    ui->setupUi(this);
    showImage(fileName);
    defaultContextMenuSetup();
    connect(ui->graphicsView, &QGraphicsView::customContextMenuRequested, this, &ImageViewerModule::onCustomContextMenuRequested);
    connect(&scene, &CustomGraphicsScene::mouseMoved, this, &ImageViewerModule::onMouseMoved);
    connect(&scene, &CustomGraphicsScene::dragStarted, this, &ImageViewerModule::onDragStarted);
    connect(&scene, &CustomGraphicsScene::dragStopped, this, &ImageViewerModule::onDragStopped);
    connect(&scene, &CustomGraphicsScene::doubleCLick, this, &ImageViewerModule::onDoubleClick);
    connect(&settingsPanel, &ImageViewerSettings::setupSettingsToSelected, this, &ImageViewerModule::onSettingsSetupRequested);
    connect(&settingsPanel, &ImageViewerSettings::setupSettingsToAll, this, &ImageViewerModule::onSettingsSetupToAllRequested);
    connect(&settingsPanel, &ImageViewerSettings::deleteItemRequested, this, &ImageViewerModule::onItemDeleteRequested);
}

ImageViewerModule::~ImageViewerModule()
{
    for(int key : imageViewerItems.keys()){
        ImageViewerItem* item = imageViewerItems[key];
        delete item;
    }

    delete ui;
}

QWidget *ImageViewerModule::toWidget()
{
    return this;
}

XmlObject ImageViewerModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::IMAGEVIEWER));
    obj.addAttribute("ImageName", fileName);

    for(int key : imageViewerItems.keys()){
        ImageViewerItem* item = imageViewerItems[key];
        obj.addChild(item->toXmlObject());
    }

    return obj;
}

void ImageViewerModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getName(ModuleId::IMAGEVIEWER)){
        QString path = xmlObject.getAttribute("ImageName");
        if(path != ""){
            this->fileName = path;
            showImage(fileName);
        }
        for(int i = 0; i < xmlObject.childCount(); i++){
            XmlObject child = xmlObject.childAt(i);
            if(child.getObjectName() == "svg"){
                addItem(child);
            }
        }
    }
}


void ImageViewerModule::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event);
    fitImage();
}

void ImageViewerModule::showImage(const QString &filename)
{
    firstResizeDone = false;
    QString filePaht = SkywardHubStrings::defaultConfigurationFolder + "/" + filename;
    QPixmap pixmap(filePaht);
    showImage(pixmap);
}

void ImageViewerModule::showImage(const QPixmap &pixmap)
{
    backgroundImg.setPixmap(pixmap);
    scene.addItem(&backgroundImg);
    ui->graphicsView->setScene(&scene);
    fitImage();
}

void ImageViewerModule::addItem(const XmlObject &xml)
{
    int itemId = onAddNewItemRequested();
    if(imageViewerItems.contains(itemId)){
        ImageViewerItem *item = imageViewerItems[itemId];
        item->setXml(xml);

        if(!item->getTopic().isEmpty()){
            onItemTopicChanged(item);
        }
        scene.updateImageViewItem(item);
    }
}

void ImageViewerModule::addCustomActionsToMenu()
{
    QAction* settings = new QAction("Change Image Name");
    connect(settings, &QAction::triggered,this, &ImageViewerModule::onChangeImageRequested);

    QAction* fitView = new QAction("Fit Image");
    connect(fitView, &QAction::triggered,this, &ImageViewerModule::fitImage);

    QAction* addItem = new QAction("Add Item");
    connect(addItem, &QAction::triggered,this, &ImageViewerModule::onAddNewItemRequested);

    QAction* showInfo = new QAction("Info");
    connect(showInfo, &QAction::triggered,this, [](){
        QMessageBox msgBox;
        msgBox.setText(SkywardHubStrings::imageViewerInfo);
        msgBox.exec();
    });

    addActionToMenu(settings);
    addActionToMenu(fitView);
    addActionToMenu(addItem);
    addActionToMenu(showInfo);
}

void ImageViewerModule::onChangeImageRequested()
{
    bool ok;
    QString text = QInputDialog::getText(this, "Select new image name",
                                         "Input a new name for the image file. Remembre to put the extention at the end of the file and put the file in the " + SkywardHubStrings::defaultConfigurationFolder + " folder" ,
                                         QLineEdit::Normal, fileName, &ok);
    if (ok && !text.isEmpty())
        onImageFilePathChanged(text);
}


void ImageViewerModule::fitImage()
{
    ui->graphicsView->fitInView(&backgroundImg, Qt::KeepAspectRatio);
}

void ImageViewerModule::onImageFilePathChanged(const QString &newFilePath)
{
    QString trimmedFilePath = newFilePath.trimmed();
    if(trimmedFilePath != ""){
        fileName = trimmedFilePath;
        showImage(fileName);
    }
}


int ImageViewerModule::onAddNewItemRequested()
{
    return addNewItem(defaultCreationPoint);
}

void ImageViewerModule::onMouseMoved(const QPointF &point)
{
    if(!firstResizeDone){
        firstResizeDone = true;

        fitImage();
    }


    for(int key : imageViewerItems.keys()){
        ImageViewerItem* item = imageViewerItems[key];
        if(item->onMouseMoved(point)){
            scene.updateImageViewItem(item);
        }
    }
}

void ImageViewerModule::onDragStarted(const QPointF &initialPosition)
{
    for(int key : imageViewerItems.keys()){
        ImageViewerItem* item = imageViewerItems[key];
        if(item->isPointInsideMyArea(initialPosition)){
            item->startDrag(initialPosition);
        }
    }
}

void ImageViewerModule::onDragStopped()
{
    for(int key : imageViewerItems.keys()){
        ImageViewerItem* item = imageViewerItems[key];
        item->stopDrag();
    }
}

void ImageViewerModule::onDoubleClick(const QPointF &point)
{
    QList<ImageViewerItem*> clickedItems;
    for(int key : imageViewerItems.keys()){
        ImageViewerItem* item = imageViewerItems[key];
        if(item->isPointInsideMyArea(point)){
            clickedItems.append(item);
        }
    }

    // Take the last selected (the items ore ordered by creation time)
    if(clickedItems.count() > 0){
        showInSettings(clickedItems[clickedItems.count() - 1]);
    }
}

int ImageViewerModule::addNewItem(const QPointF &point)
{
    return addItem(createNewItem(point));
}

int ImageViewerModule::pickUnusedItemId()
{
    while(imageViewerItems.contains(lastUsedId)){
        lastUsedId++;
    }
    return lastUsedId;
}

ImageViewerItem *ImageViewerModule::createNewItem(const QPointF &point)
{
    ImageViewerItem* newItem = new ImageViewerItem();
    newItem->setId(pickUnusedItemId());
    newItem->setCenter(point);
    return newItem;
}

int ImageViewerModule::addItem(ImageViewerItem *item)
{
    imageViewerItems[item->getId()] = item;
    scene.addImageViewItem(item);
    return item->getId();
}

void ImageViewerModule::showInSettings(ImageViewerItem *item)
{
    settingsPanel.setTarget(item->getId(), item->toXmlObject());
    settingsPanel.show();
}

void ImageViewerModule::onSettingsSetupRequested(int id)
{
    if(imageViewerItems.contains(id)){
        ImageViewerItem* item = imageViewerItems[id];
        QString previousTopic = item->getTopic();
        item->setXml(settingsPanel.getDefaultSettings());
        scene.updateImageViewItem(item);

        if(item->getTopic() != previousTopic){
            onItemTopicChanged(item);
        }
    }
}

void ImageViewerModule::onSettingsSetupToAllRequested()
{
    for(int key : imageViewerItems.keys()){
        ImageViewerItem* item = imageViewerItems[key];
        QString previousText = item->getText();
        QString previousTopic = item->getTopic();
        QPointF previousPosition = item->getTopLeftCornerPosition();
        item->setXml(settingsPanel.getDefaultSettings());
        item->setText(previousText);
        item->setTopLeftCornerPosition(previousPosition);
        item->setTopic(previousTopic);
        scene.updateImageViewItem(item);
    }
}

void ImageViewerModule::onItemTopicChanged(ImageViewerItem *item)
{
    if(topicMap.contains(item->getId())){
        QString oldTopic = topicMap[item->getId()];
        getCore()->getModuleMessagesBroker()->unsubscribe(oldTopic,this);
        topicMap.remove(item->getId());
    }

    QString newTopic = item->getTopic();
    if(newTopic != ""){
        topicMap[item->getId()] = newTopic;
        getCore()->getModuleMessagesBroker()->subscribe(newTopic,this, [this](const ModuleMessage &msg){
            this->onReceiveMsg(msg);
        });
    }
}

void ImageViewerModule::onReceiveMsg(const ModuleMessage &msg)
{
    QList<int> keys = topicMap.keys();
    for(int key : keys){
        if(msg.topic() == topicMap[key] && imageViewerItems.contains(key)){
            ImageViewerItem *item = imageViewerItems[key];
            item->onModuleMessageReceived(msg);
            scene.updateImageViewItem(item);
        }
    }
}

void ImageViewerModule::onItemDeleteRequested(int itemId)
{
    if(imageViewerItems.contains(itemId)){
        delete imageViewerItems[itemId];
        imageViewerItems.remove(itemId);
    }

    if(topicMap.contains(itemId)){
        topicMap.remove(itemId);
    }

    scene.removeImageViewItem(itemId);
}

