#include "imageviewersettings.h"
#include "ui_imageviewersettings.h"

#include <QGraphicsView>
#include <QColorDialog>
#include <QColor>

ImageViewerSettings::ImageViewerSettings(QWidget *parent) : QWidget(parent), ui(new Ui::ImageViewerSettings)
{
    ui->setupUi(this);
    defaultSettingsSetup();
    initAndConnectUI();
}

ImageViewerSettings::~ImageViewerSettings()
{
    delete ui;
}

XmlObject ImageViewerSettings::getDefaultSettings() const
{
    return defaultSettings;
}

void ImageViewerSettings::initAndConnectUI()
{
    ui->graphicsView_preview->setScene(&scene);
    scene.addItem(&preview);
    preview.setPos(10, 10);


    // Initialize values
//    QList<XmlObject*> childs = defaultSettings.deepSearchObjects("text");
//    if(!childs.isEmpty()){
//        XmlObject* text = childs[0];
//        int x, y;
//        if(text->getIntAttribute("x", x) && text->getIntAttribute("y", y)){
//            ui->doubleSpinBox_text_position_x->setValue(x);
//            ui->doubleSpinBox_text_position_y->setValue(y);
//        }
//    }


    connect(ui->plainTextEdit_preview, &QPlainTextEdit::textChanged, [this](){onPreviewTextChanged();});

    connect(ui->spinBox_width, SIGNAL(valueChanged(int)), this, SLOT(onPropertyChanged()));
    connect(ui->spinBox_height, SIGNAL(valueChanged(int)), this, SLOT(onPropertyChanged()));
    connect(ui->spinBox_border_size, SIGNAL(valueChanged(int)), this, SLOT(onPropertyChanged()));
    connect(ui->spinBox_fontSize, SIGNAL(valueChanged(int)), this, SLOT(onPropertyChanged()));
    connect(ui->doubleSpinBox_text_position_x, SIGNAL(valueChanged(double)), this, SLOT(onPropertyChanged()));
    connect(ui->doubleSpinBox_text_position_y, SIGNAL(valueChanged(double)), this, SLOT(onPropertyChanged()));

    connect(ui->checkBox_fill, &QCheckBox::toggled, this, &ImageViewerSettings::onPropertyChanged);

    connect(ui->lineEdit_text, &QLineEdit::textChanged, this, &ImageViewerSettings::onPropertyChanged);
    connect(ui->lineEdit_text_color, &QLineEdit::textChanged, this, &ImageViewerSettings::onPropertyChanged);
    connect(ui->lineEdit_fillColor, &QLineEdit::textChanged, this, &ImageViewerSettings::onPropertyChanged);
    connect(ui->lineEdit_border_color, &QLineEdit::textChanged, this, &ImageViewerSettings::onPropertyChanged);
    connect(ui->lineEdit_topic, &QLineEdit::textChanged, this, &ImageViewerSettings::onPropertyChanged);

    connect(ui->pushButton_applyToAll, &QPushButton::clicked, this, &ImageViewerSettings::onApplyToAllClicked);
    connect(ui->pushButton_applyToSelected, &QPushButton::clicked, this, &ImageViewerSettings::onApplyToSelectedClicked);
    connect(ui->pushButton_pickBorderColor, &QPushButton::clicked, this, [this](){onPickColorRequested(ui->lineEdit_border_color);});
    connect(ui->pushButton_pickTextColor, &QPushButton::clicked, this, [this](){onPickColorRequested(ui->lineEdit_text_color);});
    connect(ui->pushButton_pickFillColor, &QPushButton::clicked, this, [this](){onPickColorRequested(ui->lineEdit_fillColor);});

    connect(ui->pushButton_delete, &QPushButton::clicked, this, &ImageViewerSettings::onDeleteClicked);


    updatePreview();
}

void ImageViewerSettings::defaultSettingsSetup()
{
    defaultSettings.setObjectName("svg");
//    ui->spinBox_height->setValue(150);
//    ui->spinBox_width->setValue(150);


    XmlObject text("text");
    text.addAttribute("x",5);
    text.addAttribute("y",10);
    text.setTextValue("");

    defaultSettings.addChild(text);
}


void ImageViewerSettings::onApplyToAllClicked()
{
    emit setupSettingsToAll();
}

void ImageViewerSettings::onApplyToSelectedClicked()
{
    emit setupSettingsToSelected(target);
}

XmlObject *ImageViewerSettings::findOrAddXmlObject(const QString &xmlChildName)
{
    QList<XmlObject*> childs = defaultSettings.deepSearchObjects(xmlChildName);
    if(childs.isEmpty()){
        XmlObject c(xmlChildName);
        int childIndex = defaultSettings.addChild(c);
        return defaultSettings.getChild(childIndex);
    }
    else{
        return childs[0];
    }
}


void ImageViewerSettings::onPreviewTextChanged()
{
    lockUpdate = true;
    defaultSettings.fromXml(ui->plainTextEdit_preview->toPlainText());
    updateUI(defaultSettings);
    lockUpdate = false;
    updatePreview(false);
}


void ImageViewerSettings::onPickColorRequested(QLineEdit *lineEdit)
{
    QColor initial(lineEdit->text().trimmed());
    QColor color = QColorDialog::getColor(initial, this);
    lineEdit->setText(color.name());
}

void ImageViewerSettings::onPropertyChanged()
{
    int width = ui->spinBox_width->value();
    int height = ui->spinBox_height->value();
    QString fill = "none";
    if(ui->checkBox_fill->isChecked()){
        fill = ui->lineEdit_fillColor->text().trimmed();
    }

    defaultSettings.addAttribute("height", height);
    defaultSettings.addAttribute("width", width);

    XmlObject* text = findOrAddXmlObject("text");
    text->addAttribute("x",QString::number(ui->doubleSpinBox_text_position_x->value()));
    text->addAttribute("y",QString::number(ui->doubleSpinBox_text_position_y->value()));
    text->addAttribute("fill",ui->lineEdit_text_color->text().trimmed());
    text->addAttribute("font-size", ui->spinBox_fontSize->value());
    text->setTextValue(ui->lineEdit_text->text().trimmed());

    XmlObject* border = findOrAddXmlObject("rect");
    border->addAttribute("width", width);
    border->addAttribute("height", height);
    border->addAttribute("fill", fill);
    border->addAttribute("stroke", ui->lineEdit_border_color->text().trimmed());
    border->addAttribute("stroke-width", ui->spinBox_border_size->value());

    XmlObject* topic = findOrAddXmlObject("topic");
    topic->setTextValue(ui->lineEdit_topic->text().trimmed());

    updatePreview();
}

void ImageViewerSettings::updateUI(XmlObject &xml)
{
    bool ok;

    int width = xml.getAttribute("width").toInt(&ok);
    if(ok){
        ui->spinBox_width->setValue(width);
    }
    int height = xml.getAttribute("height").toInt(&ok);
    if(ok){
        ui->spinBox_height->setValue(height);
    }

    QList<XmlObject*> childs = xml.deepSearchObjects("text");
    if(!childs.isEmpty()){
        XmlObject *text = childs[0];
        double val = text->getAttribute("x").toDouble(&ok);
        if(ok){
            ui->doubleSpinBox_text_position_x->setValue(val);
        }
        val = text->getAttribute("y").toDouble(&ok);
        if(ok){
            ui->doubleSpinBox_text_position_y->setValue(val);
        }

        ui->lineEdit_text_color->setText(text->getAttribute("fill"));
        ui->lineEdit_text->setText(text->getTextValue());

        int fontSize = text->getAttribute("font-size").toInt(&ok);
        if(ok){
            ui->spinBox_fontSize->setValue(fontSize);
        }
    }

    childs = xml.deepSearchObjects("rect");
    if(!childs.isEmpty()){
        XmlObject *border = childs[0];
        int stroke = border->getAttribute("stroke-width").toInt(&ok);
        if(ok){
            ui->spinBox_border_size->setValue(stroke);
        }

        ui->lineEdit_border_color->setText(border->getAttribute("stroke"));
        ui->lineEdit_fillColor->setText(border->getAttribute("fill"));
    }

    childs = xml.deepSearchObjects("topic");
    if(!childs.isEmpty()){
        XmlObject *topicXml = childs[0];
        ui->lineEdit_topic->setText(topicXml->getTextValue());
    }
}

void ImageViewerSettings::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event);
    fitPreview();
}

void ImageViewerSettings::fitPreview()
{
    ui->graphicsView_preview->fitInView(&preview, Qt::KeepAspectRatio);
}

void ImageViewerSettings::onDeleteClicked()
{
    this->close();
    emit deleteItemRequested(target);
}


void ImageViewerSettings::setTarget(int value, XmlObject setup)
{
    target = value;
    updateUI(setup);
    defaultSettings = setup;
    updatePreview();
}

void ImageViewerSettings::updatePreview(bool updatePlainText)
{
    if(lockUpdate)
        return;

    QPixmap previewPixmap;
    QString xml = defaultSettings.toXml();
    previewPixmap.loadFromData(xml.toUtf8());
    preview.setPixmap(previewPixmap);
    fitPreview();
    if(updatePlainText){
        ui->plainTextEdit_preview->clear();
        ui->plainTextEdit_preview->appendPlainText(xml);
    }

    if(ui->checkBox_autoApply->isChecked()){
        onApplyToSelectedClicked();
    }

}
