#include "mavlinkmodule.h"
#include "ui_mavlinkmodule.h"
#include "Modules/Utility/contextmenuseparator.h"
#include <QDebug>
#include <QSerialPortInfo>
#include "Core/modulemessagesbroker.h"
#include "Modules/skywardhubstrings.h"
#include "rs232.h"

MavlinkModule::MavlinkModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::MavlinkModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();

    initializeSerialPortView();
//    buildCentralGraphView();
    connectUiSlots();
    qRegisterMetaType<ModuleMessage>();
    connect(&mavlinkReader,&MavlinkReader::msgReceived, this, &MavlinkModule::onMsgReceived);
    connect(&mavlinkCommandAdapter,&MavlinkCommandAdapter::publishRequested, this, &MavlinkModule::publish);
    connect(&linkQualityTimer, &QTimer::timeout, this, &MavlinkModule::onLinkQualityTimerTick);
//    mavlinkCommandAdapter.setSerialPort(&serialPort);
    subscribe();
}

MavlinkModule::~MavlinkModule()
{
    mavlinkReader.stopReading();
    delete ui;
    onStopClicked();
}

QWidget* MavlinkModule::toWidget()
{
    return this;
}

XmlObject MavlinkModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::MAVLINK));
    obj.addAttribute("BaudRateIndex",QString::number(ui->comboBox_baudRate->currentIndex()));
    return obj;
}

void MavlinkModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getName(ModuleId::MAVLINK)){
        bool ok;
        int index = xmlObject.getAttribute("BaudRateIndex").toInt(&ok);
        if(ok){
            ui->comboBox_baudRate->setCurrentIndex(index);
        }
    }
}

void MavlinkModule::subscribe()
{    
    getCore()->getModuleMessagesBroker()->subscribe(SkywardHubStrings::commandsTopic,this,[this](const ModuleMessage &msg){
        onCommandReceived(msg);
    });

    getCore()->getModuleMessagesBroker()->subscribe(SkywardHubStrings::telemetryRequestTopic,this,[this](const ModuleMessage &msg){
        onTelemetryRequestReceived(msg);
    });
}

void MavlinkModule::publish(const ModuleMessage &msg)
{
    getCore()->getModuleMessagesBroker()->publish(msg);
}

void MavlinkModule::closePort()
{
    if(portOpen && portNumber >= 0){
        // rs232 code
        portOpen = false;
        RS232_CloseComport(portNumber);
        return;
    }
//    if(serialPort.isOpen()){
//        serialPort.close();
//    }
}


void MavlinkModule::initializeSerialPortView()
{
    //fill the baud rate combo box
    QString baudPrefix = "BaudRate: ";
    ui->comboBox_baudRate->addItem(baudPrefix + "115200", QSerialPort::Baud115200);
    ui->comboBox_baudRate->addItem(baudPrefix + "57600", QSerialPort::Baud57600);
    ui->comboBox_baudRate->addItem(baudPrefix + "38400", QSerialPort::Baud38400);
    ui->comboBox_baudRate->addItem(baudPrefix + "19200", QSerialPort::Baud19200);
    ui->comboBox_baudRate->addItem(baudPrefix + "9600", QSerialPort::Baud9600);
    ui->comboBox_baudRate->addItem(baudPrefix + "4800", QSerialPort::Baud4800);
    ui->comboBox_baudRate->addItem(baudPrefix + "2400", QSerialPort::Baud2400);
    ui->comboBox_baudRate->addItem(baudPrefix + "1200", QSerialPort::Baud1200);


    bool first = true;
    //Check available port
    const auto serialPortInfos = QSerialPortInfo::availablePorts();
    for (const QSerialPortInfo &serialPortInfo : serialPortInfos) {
//        QString serialNumber = serialPortInfo.serialNumber();
//        QVariant data = (!serialNumber.isEmpty() ? serialNumber : "N/A");
        QVariant data(serialPortInfo.portName());
        ui->comboBox_serialPort->addItem(serialPortInfo.portName(), data);

        if(first){
            ui->lineEdit_serialPort->setText(serialPortInfo.portName());
            first = false;
        }
    }
}


bool MavlinkModule::startReadingOnSerialPort()
{
    QString portName = ui->lineEdit_serialPort->text().trimmed();
    bool ok;
    int baudRate = ui->comboBox_baudRate->currentData().toInt(&ok);

    QString portNumberChar = portName.remove("COM");
    std::string mode = "8N1";
    bool ok2;
    int number = portNumberChar.toInt(&ok2);
    if(ok && ok2 && !portOpen){
        portNumber = number - 1;
        if (RS232_OpenComport(portNumber,baudRate, mode.c_str())){
            QString errorMsg = "Mavlink >> Error, can't open serial port";
            qDebug() << errorMsg;
            portOpen = -1;
        }
        else{
            // All good

            return true;
        }
    }


    // QSerialport code
//    if(ok && portName != "" && !serialPort.isOpen()){
//        QSerialPortInfo port(portName);
//        serialPort.setPort(port);
//        serialPort.setBaudRate(baudRate);
////        serialPort.setDataBits(QSerialPort::Data8);
////        serialPort.setParity(QSerialPort::NoParity);
////        serialPort.setStopBits(QSerialPort::OneStop);
//        return serialPort.open(QIODevice::ReadWrite);
//    }
    return false;
}

void MavlinkModule::onStartClicked()
{
    if(startReadingOnSerialPort()){
        portOpen = true;
        mavlinkReader.setPortNumber(portNumber);

        if(ui->logEnabled_checkBox->isChecked()){
            mavlinkReader.openLogFile();
        }

        mavlinkReader.startAsyncReading();
        mavlinkCommandAdapter.setPortNumber(portNumber);
        //mavlinkReader.startAsyncReadingWithSignal(&serialPort);
    }
}

void MavlinkModule::onStopClicked()
{
    mavlinkReader.stopReading();
    closePort();
}

void MavlinkModule::onMsgReceived(const ModuleMessage &msg)
{
    msgArrived++;
    publish(msg);
}

void MavlinkModule::onStartStreamToggled(bool state)
{
    onLinkQualityIndicatorClicked(ui->linkQuality_groupBox->isChecked());

    if(state){
        onStartClicked();
    }
    else{
        onStopClicked();
    }
}

void MavlinkModule::onLinkQualityIndicatorClicked(bool checked)
{
    if(checked && !linkQualityTimer.isActive()){
        linkQualityTimer.start(linkQualityPeriod);
    }
    else if(!checked && linkQualityTimer.isActive()){
        linkQualityTimer.stop();
    }
}

void MavlinkModule::onLinkQualityTimerTick()
{
    float ratio = msgArrived/ui->spinBox_maxPackets->value();
    updateLinkSignalIndicator(msgArrived, ratio);
    msgArrived = 0;
}

void MavlinkModule::updateLinkSignalIndicator(int msgArrived, float ratio)
{
    ui->msgPerSecond_label->setText(QString::number(msgArrived));
    ModuleMessage linkQualityMsgNumber(SkywardHubStrings::mavlink_quality_link_topic + "/ReceivedMsgNumber", QString::number(msgArrived));
    ModuleMessage linkQualityMsgRatio(SkywardHubStrings::mavlink_quality_link_topic + "/ReceivedRatio", QString::number(ratio));

    publish(linkQualityMsgNumber);
    publish(linkQualityMsgRatio);
}

void MavlinkModule::onOpenLogFolderClick()
{
    QDir dir(SkywardHubStrings::defaultLogsFolder);
    if (dir.exists()){
        QDesktopServices::openUrl( QUrl::fromLocalFile(SkywardHubStrings::defaultLogsFolder) );
    }
    else{
        QString msg = "The log folder does not exist yet.\n\nIt will be created when opening the serial port.\n\n" +  SkywardHubStrings::defaultLogsFolder;
        QMessageBox msgBox;
        msgBox.setText(msg);
        msgBox.exec();
    }
}

void MavlinkModule::connectUiSlots()
{
    ui->startStop_button_layout->insertWidget(0, &startSerialStreamToggleButton);
    connect(&startSerialStreamToggleButton, &ToggleButton::toggled, this, &MavlinkModule::onStartStreamToggled);

//    connect(ui->button_start,&QPushButton::clicked,this, &MavlinkModule::onStartClicked);
//    connect(ui->button_stop,&QPushButton::clicked,this, &MavlinkModule::onStopClicked);
//    connect(ui->spinBox_maxPackets,SIGNAL(valueChanged(int)),this,SLOT(onMaxPackedValueChanged(int)));
    connect(ui->openLogFolder_button, &QPushButton::clicked, this, &MavlinkModule::onOpenLogFolderClick);
    connect(ui->comboBox_serialPort, SIGNAL(currentIndexChanged(int)), this, SLOT(onSerialPortIndexChanged(int)));
    connect(ui->linkQuality_groupBox, &QGroupBox::clicked, this, &MavlinkModule::onLinkQualityIndicatorClicked);
}

void MavlinkModule::onSerialPortIndexChanged(int index)
{
    QVariant data = ui->comboBox_serialPort->itemData(index);
    ui->lineEdit_serialPort->setText(data.toString());
}

void MavlinkModule::onCommandReceived(const ModuleMessage &msg)
{
    QString arg = msg.originalTopic().replace(SkywardHubStrings::commandsTopic+"/","");
    mavlink_message_t encoded_mvl_msg;
    if(arg == "PING_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_PING_TC();
    }
    else if(arg == "START_LAUNCH_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_START_LAUNCH_TC(msg);
    }

    else if(arg == "SET_AEROBRAKE_ANGLE_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_AEROBRAKE_ANGLE_TC(msg);
    }
    else if(arg == "SET_REFERENCE_ALTITUDE"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_REFERENCE_ALTITUDE_TC(msg);
    }
    else if(arg == "SET_REFERENCE_TEMPERATURE_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_REFERENCE_TEMPERATURE_TC(msg);
    }
    else if(arg == "SET_DEPLOYMENT_ALTITUDE_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_DEPLOYMENT_ALTITUDE_TC(msg);
    }
    else if(arg == "SET_INITIAL_ORIENTATION_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_INITIAL_ORIENTATION_TC(msg);
    }
    else if(arg == "SET_INITIAL_COORDINATES_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_INITIAL_COORDINATES_TC(msg);
    }
    else if(arg == "RAW_EVENT_TC"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_RAW_EVENT_TC(msg);
    }


    // NOARG_TC
    else if(arg == "MAV_CMD_ARM"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_ARM);
    }
    else if(arg == "MAV_CMD_DISARM"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_DISARM);
    }
    else if(arg == "MAV_CMD_FORCE_LAUNCH"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_FORCE_LAUNCH);
    }
    else if(arg == "MAV_CMD_CALIBRATE_SENSORS"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_CALIBRATE_SENSORS);
    }
    else if(arg == "MAV_CMD_CALIBRATE_ALGOS"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_CALIBRATE_ALGOS);
    }
    else if(arg == "MAV_CMD_FORCE_INIT"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_FORCE_INIT);
    }
    else if(arg == "MAV_CMD_NOSECONE_OPEN"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_NOSECONE_OPEN);
    }
    else if(arg == "MAV_CMD_DPL_WIGGLE_SERVO"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_DPL_WIGGLE_SERVO);
    }
    else if(arg == "MAV_CMD_ARB_WIGGLE_SERVO"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_ARB_WIGGLE_SERVO);
    }
    else if(arg == "MAV_CMD_DPL_RESET_SERVO"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_DPL_RESET_SERVO);
    }
    else if(arg == "MAV_CMD_ARB_RESET_SERVO"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_ARB_RESET_SERVO);
    }
    else if(arg == "MAV_CMD_CUT_DROGUE"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_CUT_DROGUE);
    }
//    else if(arg == "MAV_CMD_CUT_PRIMARY"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_CUT_PRIMARY);
//    }
//    else if(arg == "MAV_CMD_CUT_BACKUP"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_CUT_BACKUP);
//    }
//    else if(arg == "MAV_CMD_TEST_PRIMARY_CUTTER"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_TEST_PRIMARY_CUTTER);
//    }
//    else if(arg == "MAV_CMD_TEST_BACKUP_CUTTER"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_TEST_BACKUP_CUTTER);
//    }
    else if(arg == "MAV_CMD_START_LOGGING"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_START_LOGGING);
    }
//    else if(arg == "MAV_CMD_STOP_LOGGING"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_STOP_LOGGING);
//    }
    else if(arg == "MAV_CMD_CLOSE_LOG"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_CLOSE_LOG);
    }
    else if(arg == "MAV_CMD_TEST_AEROBRAKES"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_TEST_AEROBRAKES);
    }
    else if(arg == "MAV_CMD_DISABLE_AEROBRAKES"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_DISABLE_AEROBRAKES);
    }
    else if(arg == "MAV_CMD_END_MISSION"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_END_MISSION);
    }
    else if(arg == "MAV_CMD_BOARD_RESET"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_BOARD_RESET);
    }
    else if(arg == "MAV_CMD_TEST_MODE"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_TEST_MODE);
    }
    else if(arg == "MAV_CMD_RADIO_TM"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_RADIO_TM);
    }
    else if(arg == "MAV_CMD_SERIAL_TM"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_SERIAL_TM);
    }
//    else if(arg == "MAV_CMD_WIGGLE_SERVO"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_WIGGLE_SERVO);
//    }
//    else if(arg == "MAV_CMD_RESET_SERVO"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_RESET_SERVO);
//    }
//    else if(arg == "MAV_CMD_CALIBRATE_ADA"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_NOARG_TC(MAV_CMD_CALIBRATE_ADA);
//    }

//    else if(arg == "MAV_SET_DEPLOYMENT_ALTITUDE"){
//        float setting_value;
//        XmlObject msgOption = msg.getOptions();
//        int temp;
//        if(msgOption.getIntAttribute(SkywardHubStrings::upload_settings_value, temp)){
//            setting_value = temp;
//            encoded_mvl_msg = mavlinkCommandAdapter.encode_UPLOAD_SETTING_TC(MAV_SET_DEPLOYMENT_ALTITUDE_TC, setting_value);
//        }
//    }
//    else if(arg == "MAV_SET_REFERENCE_TEMP"){
//        float setting_value;
//        XmlObject msgOption = msg.getOptions();
//        int temp;
//        if(msgOption.getIntAttribute(SkywardHubStrings::upload_settings_value, temp)){
//            setting_value = temp;
//            encoded_mvl_msg = mavlinkCommandAdapter.encode_UPLOAD_SETTING_TC(MAV_SET_REFERENCE_TEMP, setting_value);
//        }
//    }
//    else if(arg == "MAV_SET_REFERENCE_ALTITUDE"){
//        float setting_value;
//        XmlObject msgOption = msg.getOptions();
//        int temp;
//        if(msgOption.getIntAttribute(SkywardHubStrings::upload_settings_value, temp)){
//            setting_value = temp;
//            encoded_mvl_msg = mavlinkCommandAdapter.encode_UPLOAD_SETTING_TC(MAV_SET_REFERENCE_ALTITUDE, setting_value);
//        }
//    }

    //encoded_mvl_msg.sysid = validSysid;
    //encoded_mvl_msg.compid = validCompid;

    if(portOpen && portNumber >= 0){
        mavlinkCommandAdapter.send(encoded_mvl_msg);
    }

}

void MavlinkModule::onTelemetryRequestReceived(const ModuleMessage &msg)
{
    QString arg = msg.originalTopic().replace(SkywardHubStrings::telemetryRequestTopic+"/","");
    mavlink_message_t encoded_mvl_msg;
    if(arg == "MAV_SYS_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_SYS_TM_ID);
    }
    else if(arg == "MAV_FMM_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_FMM_TM_ID);
    }
    else if(arg == "MAV_LOGGER_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_LOGGER_TM_ID);
    }
    else if(arg == "MAV_TMTC_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_TMTC_TM_ID);
    }
    else if(arg == "MAV_DPL_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_DPL_TM_ID);
    }
    else if(arg == "MAV_ADA_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_ADA_TM_ID);
    }
//    else if(arg == "MAV_CAN_TM_ID"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_CAN_TM_ID);
//    }
    else if(arg == "MAV_ADC_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_ADC_TM_ID);
    }
    else if(arg == "MAV_GPS_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_GPS_TM_ID);
    }
    else if(arg == "MAV_HR_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_HR_TM_ID);
    }
    else if(arg == "MAV_LR_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_LR_TM_ID);
    }
    else if(arg == "MAV_TEST_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_TEST_TM_ID);
    }
    else if(arg == "MAV_WINDTUNNEL_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_WINDTUNNEL_TM_ID);
    }
    else if(arg == "MAV_SENSORS_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_SENSORS_TM_ID);
    }

    else if(arg == "MAV_PIN_OBS_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_PIN_OBS_TM_ID);
    }
    else if(arg == "MAV_TASK_STATS_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_TASK_STATS_TM_ID);
    }
    else if(arg == "MAV_ABK_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_ABK_TM_ID);
    }
    else if(arg == "MAV_NAS_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_NAS_TM_ID);
    }
    else if(arg == "MAV_MS5803_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_MS5803_TM_ID);
    }
    else if(arg == "MAV_BMX160_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_BMX160_TM_ID);
    }
    else if(arg == "MAV_LIS3MDL_TM_ID"){
        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_LIS3MDL_TM_ID);
    }
//    else if(arg == "MAV_STRAIN_BOARD_TM_ID"){
//        encoded_mvl_msg = mavlinkCommandAdapter.encode_TELEMETRY_REQUEST_TC(MAV_STRAIN_BOARD_TM_ID);
//    }

    //encoded_mvl_msg.sysid = validSysid;
    //encoded_mvl_msg.compid = validCompid;

    if(portOpen && portNumber >= 0){
        mavlinkCommandAdapter.send(encoded_mvl_msg);
    }
}
