#include "splittermodule.h"
#include "ui_splittermodule.h"

#include <QLabel>
#include "Modules/Utility/contextmenuseparator.h"
#include <QDebug>
#include "Core/modulesmanager.h"

SplitterModule::SplitterModule(Qt::Orientation orientation) : DefaultModule(), ui(new Ui::SplitterModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    buildUI(orientation);
}

SplitterModule::~SplitterModule()
{
    delete ui;
    //qDebug() << "Splitter >> deleted";
    for(int i = 0; i < childList.count(); i++){
        if(childList[i]!= nullptr)
            delete childList[i];
    }
}

QWidget *SplitterModule::toWidget()
{
    return this;
}

XmlObject SplitterModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::SPLITTER));
    QString orientation;
    if(splitter.orientation() == Qt::Horizontal){
        orientation = "Horizontal";
    }
    else
        orientation = "Vertical";

    obj.addAttribute("Orientation", orientation);

    QString sizesString = "";
    for(int i : splitter.sizes()){
        sizesString.append(QString::number(i)+",");
    }
    obj.addAttribute("Sizes", sizesString);

    for(Module *module : childList){
        XmlObject xmlChild = module->toXmlObject();
        obj.addChild(xmlChild);
    }

    return obj;
}

void SplitterModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getCore()->getModulesManager()->getModuleName(ModuleId::SPLITTER)){
        QString orientation = xmlObject.getAttribute("Orientation");
        if(orientation != ""){
            if(orientation == "Horizontal")
                setOrientation(Qt::Horizontal);
            else
                setOrientation(Qt::Vertical);
        }

        QStringList sizes = xmlObject.getAttribute("Sizes").split(",");
        bool ok;
        QList<int> intSizes;
        for(QString size : sizes){
            int val = size.toInt(&ok);
            if(ok)
                intSizes.append(val);
        }

        int defaultChildRemaining = childList.count();

        for(int i = 0; i < xmlObject.childCount(); i++){
            XmlObject xmlChild = xmlObject.childAt(i);
            Module *module = getCore()->getModulesManager()->instantiateModuleByName(xmlChild.getObjectName());
            if(module != nullptr){
                module->fromXmlObject(xmlChild);
            }
            else{
                module = getCore()->getModulesManager()->instantiateDefaultModule();
            }
            addModule(module);
        }

        while(defaultChildRemaining > 0){
            delete childList.at(0);
            defaultChildRemaining--;
        }

        if(!intSizes.isEmpty())
            splitter.setSizes(intSizes);
    }
}

void SplitterModule::initialize(const XmlObject &params)
{
    int orientation;
    if(params.getIntAttribute("Orientation", orientation)){
        if(orientation == Qt::Vertical){
            setOrientation(Qt::Vertical);
        }
        else
            setOrientation(Qt::Horizontal);
    }
}

void SplitterModule::addModule(Module *module)
{
    int count = splitter.count();
    addModule(module,count);
}

void SplitterModule::addModule(Module *module, int position)
{
    connect(module->getModuleEventsHandler(),&ModuleEventsHandler::replaceMeWith,this,[this](Module* sender, Module* newModule){
        this->replace(sender,newModule);
    });
    connect(module->getModuleEventsHandler(),&ModuleEventsHandler::contextMenuRequest,this,&SplitterModule::onSkywardHubContextMenuRequested);
    connect(module->getModuleEventsHandler(),&ModuleEventsHandler::beforeDelete,this,&SplitterModule::onModuleDelete);

    if(position < childList.count())
        childList.insert(position, module);
    else
        childList.append(module);
    splitter.insertWidget(position,module->toWidget());
}

void SplitterModule::replace(Module *oldModule, Module *newModule)
{
    if(oldModule != nullptr && newModule != nullptr){
        int i = splitter.indexOf(oldModule->toWidget());
        addModule(newModule,i);
        //oldModule->toWidget()->deleteLater();
    }
}

void SplitterModule::setOrientation(Qt::Orientation orientation)
{
    splitter.setOrientation(orientation);
}

Qt::Orientation SplitterModule::getOrientation() const
{
    return splitter.orientation();
}

void SplitterModule::swapOrientation()
{
    if(getOrientation() == Qt::Horizontal)
        setOrientation(Qt::Vertical);
    else
        setOrientation(Qt::Horizontal);
}

void SplitterModule::buildUI(Qt::Orientation orientation)
{
    setOrientation(orientation);
//    splitter.setStyleSheet("QSplitter::handle:horizontal {"
//                           "background: qlineargradient(x1:0, y1:0, x2:1, y2:1,stop:0 #eee, stop:1 #ccc);"
//                           "border: 1px solid #777;"
//                           "width: 13px;"
//                           "margin-top: 2px;"
//                           "margin-bottom: 2px;"
//                           "border-radius: 4px;"
//                           "}");

    splitter.setChildrenCollapsible(false);
    ui->centralGrid->addWidget(&splitter);
    addModule(getCore()->getModulesManager()->instantiateDefaultModule());
    addModule(getCore()->getModulesManager()->instantiateDefaultModule());
}

void SplitterModule::addCustomActionsToMenu()
{
    showMenuActionsInSeparatedMenu("Splitter");
    QAction* split = new QAction("Split");
    connect(split,&QAction::triggered,this,[this](){
        addModule(getCore()->getModulesManager()->instantiateDefaultModule());
    });

    QAction* changeOrientation = new QAction("Swap Orientation");
    connect(changeOrientation,&QAction::triggered,this,[this](){
        this->swapOrientation();
    });

    addActionToMenu(split);
    addActionToMenu(changeOrientation);
}

void SplitterModule::onModuleDelete(Module *module)
{
    int index = childList.indexOf(module);
    if(index >= 0){
        childList.removeAt(index);
    }

    if(splitter.count() == 1){
        // When this method is called, the last module has not been deleted yet, so
        // we have to use "deleteLater" to correctly chain the deletation of module and then the splitter
        //this->deleteLater();
    }
}
