#include "stateviewermodule.h"
#include "ui_stateviewermodule.h"

#include <QInputDialog>
#include <QLineEdit>
#include "Core/modulemessagesbroker.h"

enum states {INIT = 1,
             INIT_DONE = 2,
             INIT_ERROR = 3,
             TEST_MODE = 8,
             SENSORS_CALIBRATION = 4,
             ALGOS_CALIBRATION = 5,
             DISARMED = 6,
             ARMED = 7,
             ASCENDING = 10,
             DROGUE_DESCENT = 11,
             TERMINAL_DESCENT = 12,
             LANDED = 13
            };

StateViewerModule::StateViewerModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::StateViewerModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();

    labels.append(ui->label_1_INIT);
    labels.append(ui->label_2_TEST);
    labels.append(ui->label_3_CALIBRATION);
    labels.append(ui->label_4_DISARMED);
    labels.append(ui->label_5_ARMED);
    labels.append(ui->label_6_ASCENDING);
    labels.append(ui->label_7_DROGUE);
    labels.append(ui->label_8_TERMINAL);
    labels.append(ui->label_9_END);


    QMap<int, int>indexes{
        {INIT , 0},
        {INIT_ERROR , 0},
        {INIT_DONE , 0},
        {TEST_MODE , 1},
        {SENSORS_CALIBRATION , 2},
        {ALGOS_CALIBRATION , 2},
        {DISARMED , 3 },
        {ARMED , 4},
        {ASCENDING , 5},
        {DROGUE_DESCENT , 6},
        {TERMINAL_DESCENT , 7},
        {LANDED , 8},
    };
    labelsIndexes = indexes;
}

StateViewerModule::~StateViewerModule()
{
    delete ui;
}

QWidget *StateViewerModule::toWidget()
{
    return this;
}

XmlObject StateViewerModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::STATEVIEWER));

    obj.addAttribute("topic", inputTopic);

    return obj;
}

void StateViewerModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getName(ModuleId::STATEVIEWER)){

        inputTopic = xmlObject.getAttribute("topic");
        if(inputTopic != ""){
            setTopic(inputTopic);
        }
    }
}

void StateViewerModule::addCustomActionsToMenu()
{
    QAction* config = new QAction("Configure");
    connect(config, &QAction::triggered,this, &StateViewerModule::onConfigureClicked);

    addActionToMenu(config);
}

void StateViewerModule::onConfigureClicked()
{
    bool ok;
    QString text = QInputDialog::getText(this, tr("Topic"),
                   tr("Insert input topic with the state information"), QLineEdit::Normal, inputTopic, &ok);
    text = text.trimmed();
    if (ok && !text.isEmpty())
        setTopic(text);
}

void StateViewerModule::setTopic(const QString &topic)
{
    getCore()->getModuleMessagesBroker()->unsubscribe(inputTopic, this);

    inputTopic = topic;
    getCore()->getModuleMessagesBroker()->subscribe(inputTopic, this, [this](const ModuleMessage &msg){
        onMsgReceived(msg);
    });
}

void StateViewerModule::onMsgReceived(const ModuleMessage &msg)
{
    int value = -1;
    if(msg.getIntPayload(value)){
        setState(value);
    }
}

void StateViewerModule::setState(int state)
{
    if(state != currentState){
        currentState = state;
        updateView(state);
    }
}

void StateViewerModule::updateView(int state)
{
    QString currentStateStyle = "background-color:yellow; color:black;";
    QString completedStyle = "background-color:green;";
    QString errorStyle = "background-color:red;";


    if(state == INIT){
        ui->label_1_INIT->setStyleSheet(currentStateStyle);
    }
    else if(state == INIT_DONE){
        ui->label_1_INIT->setStyleSheet(completedStyle);
    }
    else if(state == INIT_ERROR){
        ui->label_1_INIT->setStyleSheet(errorStyle);
    }
    else if(state > INIT_ERROR && labelsIndexes.contains(state)){
        int index = labelsIndexes[state];
        if(index < labels.count()){
            labels[index]->setStyleSheet(currentStateStyle);
        }
    }

    if(labelsIndexes.contains(state)){
        setStyleAfter(labelsIndexes[state]+1, "");
        setStyleBefore(labelsIndexes[state]-1, completedStyle);
    }
}

void StateViewerModule::setStyleAfter(int threshold, const QString &style)
{
    for (int i = threshold; i < labels.count(); i++ ) {
        labels[i]->setStyleSheet(style);
    }
}

void StateViewerModule::setStyleBefore(int threshold, const QString &style)
{
    for (int i = threshold; i >= 0 && i < labels.count(); i-- ) {
        labels[i]->setStyleSheet(style);
    }
}




