#include "tablemodule.h"
#include "ui_tablemodule.h"
#include "Modules/Utility/contextmenuseparator.h"
#include "Core/modulemessagesbroker.h"
#include "Modules/Utility/subscriptionspanel.h"
#include <QInputDialog>
#include <QDebug>

TableModule::TableModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::TableModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    onColumnNumberChanged(ui->column_counter->value());
    connectUiSlots();
}

TableModule::~TableModule()
{
    delete ui;
    //qDebug() << "TableModule >> Deleted";
}

QWidget *TableModule::toWidget()
{
    return this;
}

XmlObject TableModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::TABLE));

    obj.addAttribute("Column", QString::number(ui->column_counter->value()));

    for(int i = 0; i < subscriptions.count();i++){
        XmlObject subscription("Subscription");
        subscription.setTextValue(subscriptions[i]);
        obj.addChild(subscription);
    }
    return obj;
}

void TableModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getName(ModuleId::TABLE)){

        int column;
        if(xmlObject.getIntAttribute("Column", column)){
            onColumnNumberChanged(column);
        }

        for(int i = 0; i < xmlObject.childCount(); i++){
            XmlObject child = xmlObject.childAt(i);
            if(child.getObjectName() == "Subscription" && child.getTextValue() != ""){
                onSubscriptionAdded(child.getTextValue());
            }
        }
    }
}

void TableModule::onSubscribeClicked()
{
    SubscriptionsPanel *sPanel = new SubscriptionsPanel(&subscriptions);
    sPanel->setWindowTitle("Table Subscriptions");
    connect(sPanel, &SubscriptionsPanel::SubscriptionAdded, this, &TableModule::onSubscriptionAdded);
    connect(sPanel, &SubscriptionsPanel::SubscriptionRemoved, this, &TableModule::onSubscriptionRemoved);
    sPanel->show();
}

void TableModule::onSubscriptionAdded(const QString &subscription)
{
    subscriptions.append(subscription);
    getCore()->getModuleMessagesBroker()->subscribe(subscription,this,[this](const ModuleMessage &msg){
        onMsgReceived(msg);
    });
    updateVerticalHeaders(subscription);
}

void TableModule::onSubscriptionRemoved(const QString &subscription)
{
    subscriptions.removeOne(subscription);
    getCore()->getModuleMessagesBroker()->unsubscribe(subscription, this);
    removeRow(subscription);
}

void TableModule::removeRow(const QString &txt)
{
    int row = verticalHeaders.indexOf(txt);
    if(row >= 0 ){
        verticalHeaders.removeAt(row);
        ui->tableWidget->removeRow(row);
    }
}

void TableModule::onMsgReceived(const ModuleMessage &msg)
{
    int row = updateVerticalHeaders(msg.originalTopic());
    int column = updateHorizontalHeaders(msg.timestamp().toString(horizontalHeaderDateFormat));

    QTableWidgetItem *newItem = new QTableWidgetItem(msg.payload());
    ui->tableWidget->setItem(row,column,newItem);
}

int TableModule::updateVerticalHeaders(const QString &txt)
{
    int row = verticalHeaders.indexOf(txt);
    if(row >= 0){
        return row;
    }
    else{
        verticalHeaders.append(txt);
        row = verticalHeaders.count() - 1;
        ui->tableWidget->insertRow(row);
        QTableWidgetItem *header = new QTableWidgetItem(txt);
        ui->tableWidget->setVerticalHeaderItem(row, header);
        return row;
    }
}

int TableModule::updateHorizontalHeaders(const QString &txt)
{
    int column = horizontalHeaders.indexOf(txt);
    if(column >= 0){
        return column;
    }
    else{
        horizontalHeaders.insert(0,txt);
        column = 0;
        ui->tableWidget->insertColumn(column);
        QTableWidgetItem *header = new QTableWidgetItem(txt);
        ui->tableWidget->setHorizontalHeaderItem(column,header);

        // Siccome è stata inserita una colonna, viene rimossa l'ultima per mantenere il columnCount
        ui->tableWidget->removeColumn(ui->tableWidget->columnCount()-1);
        if(horizontalHeaders.count() > ui->tableWidget->columnCount()){
            horizontalHeaders.removeLast();
        }
        return column;
    }
}



void TableModule::addCustomActionsToMenu()
{
    QAction* subscribe = new QAction("Subscribe");
    connect(subscribe, &QAction::triggered,this, &TableModule::onSubscribeClicked);

    addActionToMenu(subscribe);
}

void TableModule::onColumnNumberChanged(int columnNumber)
{
    ui->tableWidget->setColumnCount(columnNumber);
    while(horizontalHeaders.count() > columnNumber && columnNumber >= 0){
        horizontalHeaders.removeLast();
    }

    if(ui->column_counter->value() != columnNumber){
        ui->column_counter->setValue(columnNumber);
    }
}

void TableModule::onTableTitleRenameRequest()
{
    bool ok;
    QString tableName = QInputDialog::getText(this, "Config name","Insert the name for this Table",QLineEdit::Normal, QString(),&ok);

    if(ok && !tableName.isEmpty()){
        ui->label_tableTitle->setText(tableName);
    }
}

void TableModule::connectUiSlots()
{
//    connect(ui->button_test, &QPushButton::clicked, this, [this](){
//        //this->getManager()->publish("Table/T1", "TableModule >> ciao ciao");
//        Message m("Table/T1","Bellaa", QDateTime::currentDateTime());
//        getManager()->publish(m);
//    });

    connect(ui->column_counter,SIGNAL(valueChanged(int)), this, SLOT(onColumnNumberChanged(int)));
    connect(ui->label_tableTitle, &QLabel::customContextMenuRequested, this, &TableModule::onTableTitleRenameRequest);
//    connect(ui->lineEdit_dateFormat, &QLineEdit::editingFinished, this, [this](){
//        horizontalHeaderDateFormat = ui->lineEdit_dateFormat->text();
//    });
}
