#include "valuesconverterviewermodule.h"
#include "ui_valuesconverterviewermodule.h"

#include "valuesviewerconfigpanel.h"
#include "Core/modulemessagesbroker.h"

ValuesConverterViewerModule::ValuesConverterViewerModule(QWidget *parent) : DefaultModule(parent), ui(new Ui::ValuesConverterViewerModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
}

ValuesConverterViewerModule::~ValuesConverterViewerModule()
{
    clearLabels();
    delete ui;
}


QWidget *ValuesConverterViewerModule::toWidget()
{
    return this;
}

XmlObject ValuesConverterViewerModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::VALUESCONVERTERVIEWER));

    obj.addAttribute("Columns", QString::number(columns));

    for(int i = 0; i < rules.count();i++){
        obj.addChild(rules[i].toXmlObject());
    }
    return obj;
}

void ValuesConverterViewerModule::fromXmlObject(const XmlObject &xmlObject)
{
    if(xmlObject.getObjectName() == getName(ModuleId::VALUESCONVERTERVIEWER)){

        int col;
        if(xmlObject.getIntAttribute("Columns", col)){
            columns = col;
        }

        for(int i = 0; i < xmlObject.childCount(); i++){
            XmlObject child = xmlObject.childAt(i);
            ValueElement el;
            if(el.fromXmlObject(child)){
                addRule(el);
            }
        }
        createLabels();
    }
}

void ValuesConverterViewerModule::onConfigureClicked()
{
    ValuesViewerConfigPanel *sPanel = new ValuesViewerConfigPanel();
    sPanel->setRules(rules);
    sPanel->setColumnsCount(columns);
    connect(sPanel, &ValuesViewerConfigPanel::configurationSaved, this, &ValuesConverterViewerModule::onConfigurationSaved);
    sPanel->show();
}

void ValuesConverterViewerModule::onConfigurationSaved(ValuesViewerConfigPanel *sPanel)
{
    setRules(sPanel->getRules());
    columns = sPanel->getColumnsCount();

    createLabels();
}

void ValuesConverterViewerModule::addCustomActionsToMenu()
{
    QAction* configure = new QAction("Configure");
    connect(configure, &QAction::triggered,this, &ValuesConverterViewerModule::onConfigureClicked);
    addActionToMenu(configure);
}

QLabel *ValuesConverterViewerModule::createView(const ValueElement &el) const
{
    QLabel *label = new QLabel();
    label->setText(el.getName());
    label->setAlignment(Qt::AlignCenter);
    return label;
}

void ValuesConverterViewerModule::clearLabels()
{
    while(labels.count() > 0){
        QLabel *l = labels[0];
        labels.removeAll(l);
        delete l;
    }
}

void ValuesConverterViewerModule::createLabels()
{
    clearLabels();
    if(columns <= 0){
        columns = minColNumber;
    }

    int col = 0;
    int row = 0;
    for (int i = 0; i < rules.count(); i++ ) {
        QLabel *label;

        int labelIndex = findLabelIndexByName(rules[i].getName());
        if(labelIndex >= 0 && labelIndex < labels.count()){
            label = labels[labelIndex];
        }
        else{
            label = createView(rules[i]);
            if(col == columns){
                col = 0;
                row++;
            }

            if(rules[i].isDisplayInView()){
                ui->mainLayout_grid->addWidget(label, row, col);
                col++;
            }
        }
        labels.append(label);

    }
}

void ValuesConverterViewerModule::addRule(const ValueElement &el)
{
    if(el.getTopic() != "" && !isTopicPresent(el.getTopic())){
        getCore()->getModuleMessagesBroker()->subscribe(el.getTopic(), this, [this](const ModuleMessage &msg){
            onMsgReceived(msg);
        });
    }
    rules.append(el);
}

void ValuesConverterViewerModule::setRules(const QList<ValueElement> &rList)
{
    clearRules();
    for (int i = 0; i < rList.count(); i++ ) {
        addRule(rList[i]);
    }
}

void ValuesConverterViewerModule::onMsgReceived(const ModuleMessage &msg)
{
    for (int i = 0; i < rules.count(); i++ ) {
        if(rules[i].getTopic() == msg.topic()){
            if(rules[i].updateCurrentValue(msg.payload())){
                QString time = "(" + msg.timestamp().toString("HH:mm:ss") + ")\n";
                labels[i]->setText(rules[i].getName()+"\n"+time+rules[i].getCurrentValue());
                if(rules[i].getColor() != ""){
                    labels[i]->setStyleSheet("color:"+rules[i].getColor()+";");
                }

                if(rules[i].isEmitActive()){
                    ModuleMessage convertedMsg(msg);
                    convertedMsg.setPayload(rules[i].getCurrentValue());
                    convertedMsg.setTopic(rules[i].getOutpuTopic());
                    getCore()->getModuleMessagesBroker()->publish(convertedMsg);
                }
            }
        }
    }
}

void ValuesConverterViewerModule::clearRules()
{
    for (int i = 0; i < rules.count(); i++ ) {
        if(rules[i].getTopic() != ""){
            getCore()->getModuleMessagesBroker()->unsubscribe(rules[i].getTopic(), this);
        }
    }
    rules.clear();
}

int ValuesConverterViewerModule::findLabelIndexByName(const QString &name)
{
    for (int i = 0; i < labels.count(); i++ ) {
        if(labels[i]->text() == name){
            return i;
        }
    }
    return -1;
}

bool ValuesConverterViewerModule::isTopicPresent(const QString &topic)
{
    for (int i = 0; i < rules.count(); i++ ) {
        if(rules[i].getTopic() == topic){
            return true;
        }
    }
    return false;
}
